==> eyflutter_uikit-0.0.8%2B8.tar.gz: lib/uikit/options/pop_widget.dart <==
import 'dart:io';
import 'dart:math' as math;

import 'package:eyflutter_core/provider/cus_change_notifier.dart';
import 'package:flutter/material.dart';

enum PressType {
  longPress,
  singleClick,
}

mixin OnPopWidgetListener {
  void onForcedShowMenu();

  void onForcedHideMenu();

  void onForcedToggleMenu();
}

class PopWidgetController extends CusChangeNotifier {
  bool menuIsShowing = false;

  void showMenu() {
    menuIsShowing = true;
    notifierLazy();
  }

  void hideMenu() {
    menuIsShowing = false;
    notifierLazy();
  }

  void toggleMenu() {
    menuIsShowing = !menuIsShowing;
    notifierLazy();
  }
}

class PopWidget extends StatefulWidget {
  final Widget child;
  final PressType pressType;
  final bool showArrow;
  final Color arrowColor;
  final Color barrierColor;
  final double horizontalMargin;
  final double verticalMargin;
  final double arrowSize;
  final PopWidgetController? controller;
  final Widget Function() menuBuilder;
  final Widget? arrowUp;

  /// pop视图显示回调
  final Function()? onPopShowCall;

  /// pop视图隐藏回调
  final Function()? onPopHideCall;

  PopWidget({
    required this.child,
    required this.menuBuilder,
    required this.pressType,
    this.controller,
    this.arrowColor = const Color(0xFF4C4C4C),
    this.showArrow = true,
    this.barrierColor = Colors.black12,
    this.arrowSize = 10.0,
    this.horizontalMargin = 10.0,
    this.verticalMargin = 10.0,
    this.onPopShowCall,
    this.onPopHideCall,
    this.arrowUp,
  });

  @override
  _PopWidgetState createState() => _PopWidgetState();
}

class _PopWidgetState extends State<PopWidget> {
  RenderBox? _childBox;
  RenderBox? _parentBox;
  OverlayEntry? _overlayEntry;
  PopWidgetController? _controller;

  _showMenu() {
    Widget arrow = widget.arrowUp ??
        ClipPath(
          child: Container(
            width: widget.arrowSize,
            height: widget.arrowSize,
            color: widget.arrowColor,
          ),
          clipper: _ArrowClipper(),
        );
    _overlayEntry = OverlayEntry(
      builder: (context) {
        return buildOverlayWidget(arrow);
      },
    );
    if (_overlayEntry != null) {
      Overlay.of(context)?.insert(_overlayEntry!);
    }
    if (widget.onPopShowCall != null) {
      widget.onPopShowCall!();
    }
  }

  Widget buildOverlayWidget(Widget arrow) {
    return Stack(
      children: <Widget>[
        GestureDetector(
          onTap: () => _hideMenu(),
          child: Container(
            color: widget.barrierColor,
          ),
        ),
        Center(
          child: Container(
            constraints: BoxConstraints(
              maxWidth: _parentBox?.size.width ?? 0 - 2 * widget.horizontalMargin,
              minWidth: 0,
            ),
            child: buildMultiChild(arrow),
          ),
        ),
      ],
    );
  }

  Widget buildMultiChild(Widget arrow) {
    return CustomMultiChildLayout(
      delegate: _MenuLayoutDelegate(
        anchorSize: _childBox?.size ?? Size.zero,
        anchorOffset: _childBox?.localToGlobal(
              Offset(-widget.horizontalMargin, 0),
            ) ??
            Offset.zero,
        verticalMargin: widget.verticalMargin,
      ),
      children: <Widget>[
        if (widget.showArrow)
          LayoutId(
            id: _MenuLayoutId.arrow,
            child: arrow,
          ),
        if (widget.showArrow)
          LayoutId(
            id: _MenuLayoutId.downArrow,
            child: Transform.rotate(
              angle: math.pi,
              child: arrow,
            ),
          ),
        LayoutId(
          id: _MenuLayoutId.content,
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Material(
                child: widget.menuBuilder(),
                color: Colors.transparent,
              ),
            ],
          ),
        ),
      ],
    );
  }

  _hideMenu() {
    if (widget.onPopHideCall != null) {
      widget.onPopHideCall!();
    }
    if (_overlayEntry != null) {
      _overlayEntry?.remove();
      _overlayEntry = null;
    }
  }

  _updateView() {
    if (_controller?.menuIsShowing ?? false) {
      _showMenu();
    } else {
      _hideMenu();
    }
  }

  @override
  void initState() {
    super.initState();
    _controller = widget.controller;
    if (_controller == null) _controller = PopWidgetController();
    _controller?.addListener(_updateView);
    WidgetsBinding.instance?.addPostFrameCallback((call) {
      _childBox = context.findRenderObject() as RenderBox;
      _parentBox = Overlay.of(context)?.context.findRenderObject() as RenderBox;
    });
  }

  @override
  void dispose() {
    _hideMenu();
    _controller?.removeListener(_updateView);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var child = Material(
      child: InkWell(
        hoverColor: Colors.transparent,
        focusColor: Colors.transparent,
        splashColor: Colors.transparent,
        highlightColor: Colors.transparent,
        child: widget.child,
        onTap: () {
          if (widget.pressType == PressType.singleClick) {
            _showMenu();
          }
        },
        onLongPress: () {
          if (widget.pressType == PressType.longPress) {
            _showMenu();
          }
        },
      ),
      color: Colors.transparent,
    );
    if (Platform.isIOS) {
      return child;
    } else {
      return WillPopScope(
        onWillPop: () {
          _hideMenu();
          return Future.value(true);
        },
        child: child,
      );
    }
  }
}

enum _MenuLayoutId {
  arrow,
  downArrow,
  content,
}

enum _MenuPosition {
  bottomLeft,
  bottomCenter,
  bottomRight,
  topLeft,
  topCenter,
  topRight,
}

class _MenuLayoutDelegate extends MultiChildLayoutDelegate {
  _MenuLayoutDelegate({
    required this.anchorSize,
    required this.anchorOffset,
    required this.verticalMargin,
  });

  final Size anchorSize;
  final Offset anchorOffset;
  final double verticalMargin;

  @override
  void performLayout(Size size) {
    Size contentSize = Size.zero;
    Size arrowSize = Size.zero;
    Offset contentOffset = Offset(0, 0);
    Offset arrowOffset = Offset(0, 0);

    double anchorCenterX = anchorOffset.dx + anchorSize.width / 2;
    double anchorTopY = anchorOffset.dy;
    double anchorBottomY = anchorTopY + anchorSize.height;
    _MenuPosition menuPosition = _MenuPosition.bottomCenter;

    if (hasChild(_MenuLayoutId.content)) {
      contentSize = layoutChild(
        _MenuLayoutId.content,
        BoxConstraints.loose(size),
      );
    }
    if (hasChild(_MenuLayoutId.arrow)) {
      arrowSize = layoutChild(
        _MenuLayoutId.arrow,
        BoxConstraints.loose(size),
      );
    }
    if (hasChild(_MenuLayoutId.downArrow)) {
      layoutChild(
        _MenuLayoutId.downArrow,
        BoxConstraints.loose(size),
      );
    }

    bool isTop = false;
    if (anchorBottomY + verticalMargin + arrowSize.height + contentSize.height > size.height) {
      isTop = true;
    }
    if (anchorCenterX - contentSize.width / 2 < 0) {
      menuPosition = isTop ? _MenuPosition.topLeft : _MenuPosition.bottomLeft;
    } else if (anchorCenterX + contentSize.width / 2 > size.width) {
      menuPosition = isTop ? _MenuPosition.topRight : _MenuPosition.bottomRight;
    } else {
      menuPosition = isTop ? _MenuPosition.topCenter : _MenuPosition.bottomCenter;
    }

    switch (menuPosition) {
      case _MenuPosition.bottomCenter:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorBottomY + verticalMargin,
        );
        contentOffset = Offset(
          anchorCenterX - contentSize.width / 2,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _MenuPosition.bottomLeft:
        arrowOffset = Offset(anchorCenterX - arrowSize.width / 2, anchorBottomY + verticalMargin);
        contentOffset = Offset(
          0,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _MenuPosition.bottomRight:
        arrowOffset = Offset(anchorCenterX - arrowSize.width / 2, anchorBottomY + verticalMargin);
        contentOffset = Offset(
          size.width - contentSize.width,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _MenuPosition.topCenter:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height,
        );
        contentOffset = Offset(
          anchorCenterX - contentSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
      case _MenuPosition.topLeft:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height,
        );
        contentOffset = Offset(
          0,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
      case _MenuPosition.topRight:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height,
        );
        contentOffset = Offset(
          size.width - contentSize.width,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
    }
    if (hasChild(_MenuLayoutId.content)) {
      positionChild(_MenuLayoutId.content, contentOffset);
    }
    bool isBottom = false;
    if (_MenuPosition.values.indexOf(menuPosition) < 3) {
      // bottom
      isBottom = true;
    }
    if (hasChild(_MenuLayoutId.arrow)) {
      positionChild(
        _MenuLayoutId.arrow,
        isBottom ? Offset(arrowOffset.dx, arrowOffset.dy + 0.1) : Offset(-100, 0),
      );
    }
    if (hasChild(_MenuLayoutId.downArrow)) {
      positionChild(
        _MenuLayoutId.downArrow,
        !isBottom ? Offset(arrowOffset.dx, arrowOffset.dy - 0.1) : Offset(-100, 0),
      );
    }
  }

  @override
  bool shouldRelayout(MultiChildLayoutDelegate oldDelegate) => false;
}

class _ArrowClipper extends CustomClipper<Path> {
  @override
  Path getClip(Size size) {
    Path path = Path();
    path.moveTo(0, size.height);
    path.lineTo(size.width / 2, size.height / 2);
    path.lineTo(size.width, size.height);
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return true;
  }
}

==> eyflutter_uikit-0.1.4%2B8.tar.gz: lib/uikit/options/pop_widget.dart <==
import 'dart:io';
import 'dart:math' as math;

import 'package:eyflutter_core/provider/cus_change_notifier.dart';
import 'package:flutter/material.dart';

enum PressType {
  longPress,
  singleClick,
}

mixin OnPopWidgetListener {
  void onForcedShowMenu();

  void onForcedHideMenu();

  void onForcedToggleMenu();
}

class PopWidgetController extends CusChangeNotifier {
  bool menuIsShowing = false;

  void showMenu() {
    menuIsShowing = true;
    notifierLazy();
  }

  void hideMenu() {
    menuIsShowing = false;
    notifierLazy();
  }

  void toggleMenu() {
    menuIsShowing = !menuIsShowing;
    notifierLazy();
  }
}

class PopWidget extends StatefulWidget {
  final Widget child;
  final PressType pressType;
  final bool showArrow;
  final Color arrowColor;
  final Color barrierColor;
  final double horizontalMargin;
  final double verticalMargin;
  final double arrowSize;
  final PopWidgetController? controller;
  final Widget Function() menuBuilder;
  final Widget? arrowUp;

  /// pop视图显示回调
  final Function()? onPopShowCall;

  /// pop视图隐藏回调
  final Function()? onPopHideCall;

  /// 剪头偏移量
  final Offset arrowOffset;

  PopWidget({
    required this.child,
    required this.menuBuilder,
    required this.pressType,
    this.controller,
    this.arrowColor = const Color(0xFF4C4C4C),
    this.showArrow = true,
    this.barrierColor = Colors.black12,
    this.arrowSize = 10.0,
    this.horizontalMargin = 10.0,
    this.verticalMargin = 10.0,
    this.onPopShowCall,
    this.onPopHideCall,
    this.arrowUp,
    this.arrowOffset = const Offset(0, 0),
  });

  @override
  _PopWidgetState createState() => _PopWidgetState();
}

class _PopWidgetState extends State<PopWidget> {
  RenderBox? _childBox;
  RenderBox? _parentBox;
  OverlayEntry? _overlayEntry;
  PopWidgetController? _controller;

  _showMenu() {
    Widget arrow = widget.arrowUp ??
        ClipPath(
          child: Container(
            width: widget.arrowSize,
            height: widget.arrowSize,
            color: widget.arrowColor,
          ),
          clipper: _ArrowClipper(),
        );
    _overlayEntry = OverlayEntry(
      builder: (context) {
        return buildOverlayWidget(arrow);
      },
    );
    if (_overlayEntry != null) {
      Overlay.of(context).insert(_overlayEntry!);
    }
    if (widget.onPopShowCall != null) {
      widget.onPopShowCall!();
    }
  }

  Widget buildOverlayWidget(Widget arrow) {
    return Stack(
      children: <Widget>[
        GestureDetector(
          onTap: () => _hideMenu(),
          child: Container(
            color: widget.barrierColor,
          ),
        ),
        Center(
          child: Container(
            constraints: BoxConstraints(
              maxWidth: _parentBox?.size.width ?? 0 - 2 * widget.horizontalMargin,
              minWidth: 0,
            ),
            child: buildMultiChild(arrow),
          ),
        ),
      ],
    );
  }

  Widget buildMultiChild(Widget arrow) {
    return CustomMultiChildLayout(
      delegate: _MenuLayoutDelegate(
        anchorSize: _childBox?.size ?? Size.zero,
        anchorOffset: _childBox?.localToGlobal(
              Offset(-widget.horizontalMargin, 0),
            ) ??
            Offset.zero,
        verticalMargin: widget.verticalMargin,
        arrow: widget.arrowOffset,
      ),
      children: <Widget>[
        if (widget.showArrow)
          LayoutId(
            id: _MenuLayoutId.arrow,
            child: arrow,
          ),
        if (widget.showArrow)
          LayoutId(
            id: _MenuLayoutId.downArrow,
            child: Transform.rotate(
              angle: math.pi,
              child: arrow,
            ),
          ),
        LayoutId(
          id: _MenuLayoutId.content,
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Material(
                child: widget.menuBuilder(),
                color: Colors.transparent,
              ),
            ],
          ),
        ),
      ],
    );
  }

  _hideMenu() {
    if (widget.onPopHideCall != null) {
      widget.onPopHideCall!();
    }
    if (_overlayEntry != null) {
      _overlayEntry?.remove();
      _overlayEntry = null;
    }
  }

  _updateView() {
    if (_controller?.menuIsShowing ?? false) {
      _showMenu();
    } else {
      _hideMenu();
    }
  }

  @override
  void initState() {
    super.initState();
    _controller = widget.controller;
    if (_controller == null) _controller = PopWidgetController();
    _controller?.addListener(_updateView);
    WidgetsBinding.instance.addPostFrameCallback((call) {
      _childBox = context.findRenderObject() as RenderBox;
      _parentBox = Overlay.of(context).context.findRenderObject() as RenderBox;
    });
  }

  @override
  void dispose() {
    _hideMenu();
    _controller?.removeListener(_updateView);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var child = Material(
      child: InkWell(
        hoverColor: Colors.transparent,
        focusColor: Colors.transparent,
        splashColor: Colors.transparent,
        highlightColor: Colors.transparent,
        child: widget.child,
        onTap: () {
          if (widget.pressType == PressType.singleClick) {
            _showMenu();
          }
        },
        onLongPress: () {
          if (widget.pressType == PressType.longPress) {
            _showMenu();
          }
        },
      ),
      color: Colors.transparent,
    );
    if (Platform.isIOS) {
      return child;
    } else {
      return WillPopScope(
        onWillPop: () {
          _hideMenu();
          return Future.value(true);
        },
        child: child,
      );
    }
  }
}

enum _MenuLayoutId {
  arrow,
  downArrow,
  content,
}

enum _MenuPosition {
  bottomLeft,
  bottomCenter,
  bottomRight,
  topLeft,
  topCenter,
  topRight,
}

class _MenuLayoutDelegate extends MultiChildLayoutDelegate {
  _MenuLayoutDelegate({
    required this.anchorSize,
    required this.anchorOffset,
    required this.verticalMargin,
    required this.arrow,
  });

  final Size anchorSize;
  final Offset anchorOffset;
  final double verticalMargin;
  final Offset arrow;

  @override
  void performLayout(Size size) {
    Size contentSize = Size.zero;
    Size arrowSize = Size.zero;
    Offset contentOffset = Offset(0, 0);
    Offset arrowOffset = Offset(arrow.dx, arrow.dy);

    double anchorCenterX = anchorOffset.dx + anchorSize.width / 2;
    double anchorTopY = anchorOffset.dy;
    double anchorBottomY = anchorTopY + anchorSize.height;
    _MenuPosition menuPosition = _MenuPosition.bottomCenter;

    if (hasChild(_MenuLayoutId.content)) {
      contentSize = layoutChild(
        _MenuLayoutId.content,
        BoxConstraints.loose(size),
      );
    }
    if (hasChild(_MenuLayoutId.arrow)) {
      arrowSize = layoutChild(
        _MenuLayoutId.arrow,
        BoxConstraints.loose(size),
      );
    }
    if (hasChild(_MenuLayoutId.downArrow)) {
      layoutChild(
        _MenuLayoutId.downArrow,
        BoxConstraints.loose(size),
      );
    }

    bool isTop = false;
    if (anchorBottomY + verticalMargin + arrowSize.height + contentSize.height > size.height) {
      isTop = true;
    }
    if (anchorCenterX - contentSize.width / 2 < 0) {
      menuPosition = isTop ? _MenuPosition.topLeft : _MenuPosition.bottomLeft;
    } else if (anchorCenterX + contentSize.width / 2 > size.width) {
      menuPosition = isTop ? _MenuPosition.topRight : _MenuPosition.bottomRight;
    } else {
      menuPosition = isTop ? _MenuPosition.topCenter : _MenuPosition.bottomCenter;
    }

    switch (menuPosition) {
      case _MenuPosition.bottomCenter:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2 + arrowOffset.dx,
          anchorBottomY + verticalMargin + arrowOffset.dy,
        );
        contentOffset = Offset(
          anchorCenterX - contentSize.width / 2,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _MenuPosition.bottomLeft:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2 + arrowOffset.dx,
          anchorBottomY + verticalMargin + arrowOffset.dy,
        );
        contentOffset = Offset(
          0,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _MenuPosition.bottomRight:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2 + arrowOffset.dx,
          anchorBottomY + verticalMargin + arrowOffset.dy,
        );
        contentOffset = Offset(
          size.width - contentSize.width,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _MenuPosition.topCenter:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2 + arrowOffset.dx,
          anchorTopY - verticalMargin - arrowSize.height + arrowOffset.dy,
        );
        contentOffset = Offset(
          anchorCenterX - contentSize.width / 2 + arrowOffset.dx,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
      case _MenuPosition.topLeft:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2 + arrowOffset.dx,
          anchorTopY - verticalMargin - arrowSize.height + arrowOffset.dy,
        );
        contentOffset = Offset(
          0,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
      case _MenuPosition.topRight:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2 + arrowOffset.dx,
          anchorTopY - verticalMargin - arrowSize.height + arrowOffset.dy,
        );
        contentOffset = Offset(
          size.width - contentSize.width,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
    }
    if (hasChild(_MenuLayoutId.content)) {
      positionChild(_MenuLayoutId.content, contentOffset);
    }
    bool isBottom = false;
    if (_MenuPosition.values.indexOf(menuPosition) < 3) {
      // bottom
      isBottom = true;
    }
    if (hasChild(_MenuLayoutId.arrow)) {
      positionChild(
        _MenuLayoutId.arrow,
        isBottom ? Offset(arrowOffset.dx, arrowOffset.dy + 0.1) : Offset(-100, 0),
      );
    }
    if (hasChild(_MenuLayoutId.downArrow)) {
      positionChild(
        _MenuLayoutId.downArrow,
        !isBottom ? Offset(arrowOffset.dx, arrowOffset.dy - 0.1) : Offset(-100, 0),
      );
    }
  }

  @override
  bool shouldRelayout(MultiChildLayoutDelegate oldDelegate) => false;
}

class _ArrowClipper extends CustomClipper<Path> {
  @override
  Path getClip(Size size) {
    Path path = Path();
    path.moveTo(0, size.height);
    path.lineTo(size.width / 2, size.height / 2);
    path.lineTo(size.width, size.height);
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return true;
  }
}
